 #include <iostream>
 #include <fstream>
 #include <cstring>
 #include <cstdlib>
 #include <iomanip>
 #include <cmath>
 #include <ctime>
 
 //---------------------------- Declaration of the main physical and mathematical constants ------------------------------------//
  const double pi  =2.0*asin(1.0);   // number pi                               //
  const double re2 =7.940787682e-26; // square of electron's radius in cm^2     //
  const double mc2 =510.99895;       // electron's mass in keV                  // 
  const double Na  =6.02214076e23;   // Avogadro's number                       //
  const double h   =6.62607015e-34;  // Planck's constant in Js units           //
  const double e   =1.602176634e-19; // electron charge in C                    //
  const double c   =2.99792458e8;    // speed of light in m/s                   //
    
 //-------- Declaration of constants specific to optimal grazing-incidence position (OGIP) XRF of trace elements in bone --------//
  const double Lc = 20.0;  // Lc  = length of Al collimator in mm //
  const double Dd = 5.6;   // detector diameter in mm             //
  const double bs = 1.7;  // bs = 1.7 mm  FWHM in mm at 15 mm distance from PXL  //
                           // bs = 0.94 mm FWHM in mm at 10 mm distance from PXL  //  
  const int    N  = 11000; // constant for all arrays             // 

 //------- Gaussian probabilities for G(x0,sigma) between a and b ---------------------------------------------------------------//
 double GaussProb (double x0, double sigma, double a, double b)
 {
   double ll   = (a-x0)/sigma;
   double ul   = (b-x0)/sigma;
   double prob = 0.5*(erf(ul/sqrt(2.0))-erf(ll/sqrt(2.0)));
   return prob;
 }
 //---- Convolution function of arrays A[N] and B[M]; return discrete convolution array element C[q] -----------------------------//
 double convol (double A[],double B[], int N, int M, int q)
 {
	double C[N+M];
	int i,j,k;
    double s=0.0;
    for (i=1;i<=(N+M-1);i++)
    {
      C[i]=0.0;
	 for (j=std::max(1-M+i,1);j<=std::min(N,i);j++)
	  {
	   k=i-j+1;
	   C[i]=C[i]+A[j]*B[k];
	  }
	s=s+C[i];
    }
	return C[q];
 }
 
 //------------------------------------------------------------ MAIN PROGRAM -----------------------------------------------------//

 using namespace std; 
 int main ()
{

/* //Verification of the numerical convolution algorithm//
  int N=3;
  int M=5;
  double A[N]; A[1]=1.0; A[2]=1.0; A[3]=2.0;
  double B[M]; B[1]=2.0; B[2]=2.0; B[3]=3.0; B[4]=3.0; B[5]=4.0;
  double C[N*M];

   for (int i=1;i<=(N+M-1);i++)
  {
    C[i]=convol(A,B,N,M,i);
	cout<<" C["<<i<<"] = "<<C[i]<<endl;
  }
 */
   
 double K[N],Ka[N],Kb[N],G[N],CKa[N],CKb[N];
 double step,yb,a,b,s,ns;
 int i,j,k,n,ncol,T,Q,M;
 
 ifstream dt;  
 dt.open("Sr_data.dat");
 if (dt.fail())
  {cout<<"The file can not be read"<<endl;}
 i=0;
 char q;
 dt.get(q);
 while (dt.good())
  {
	dt>>K[i]; 
    n=i;	 
    i++;
  }
 cout<<" The number of data points in the pencil beam XRF file is: "<<n<<endl; 
 //Placing Ka and Kb data from read file in separate arrays //
 k = 0;
 cout<<" The integer number of columns in the read file is: ncol = ";
 cin>>ncol;
 M = int(n/ncol);
 cout<<" The number of data points for each of the Ka or Kb signal columns is: M = "<<M<<endl;
 
 double time0 = time(0);
  for (i=0;i<M;i++)
 {
   Ka[i]=K[k+1];
   Kb[i]=K[k+2];
   k=k+ncol;  
 }
 step = 0.05;   //step size in mm: 0.05 for Sr and 0.0005 for Ca //
 ns   = 3.0;      //number of standard deviations //
 T    = floor(double(2.0*ns*(bs/2.355)/step));  //number of sampling points spanning ns*sigma central interval of the normal distribution //
 cout<<" The number of Gaussian distribution sample points is T = "<<T<<endl;
 a = (-1.0)*ns-double(2.0*ns/(T+1));
 s=0.0;
 for (i=0;i<=T;i++) 
 {
   a    = a + double(2.0*ns/(T+1));
   b    = a + double(2.0*ns/(T+1));
   G[i] = GaussProb(0.0,1.0,a,b);
   s=s+G[i];
 }
 cout<<" Final a = "<<a<<"; final b = "<<b<<endl;
 cout<<" G[0] = "<<G[0]<<"   G["<<int(T/2)<<"] = "<<G[T/2]<<"  G["<<T<<"] = "<<G[T]<<endl;
 cout<<" The sum of Gaussian probabilities is: "<<s<<endl;
 cout<<" The number of convolution points is: "<<T+M<<endl;

 ofstream qq;
 qq.open ("Convolution_Sr_data.dat");
 double t0 = 0.0;  //2.8 mm leather thickness in mm, 0.0 mm for bare bone //
 yb = Lc+t0+(-1.0)*ns*bs/2.355; //initial position of the microbeam center in mm with respect to detector center //
 for (i=0;i<=(T+M-1);i++)       //(i=0;i<=T;i++) //
 { 
    CKa[i]=0.0;
	CKb[i]=0.0;
	for (j=max(1-M+i,1);j<=min(T,i);j++)
	 {
	   k=i-j+1;
	   CKa[i]=CKa[i]+G[j]*Ka[k];
	   CKb[i]=CKb[i]+G[j]*Kb[k];
     }
   qq<<"  "<<yb<<" "<<i<<" "<<Ka[i]<<" "<<Kb[i]<<" "<<G[i]<<"  "<<CKa[i]<<" "<<CKb[i]<<endl;	
   yb = yb+step;
 }
 qq.close();
 
 double time1=time(0);
 cout<<" The running time of the C++ code was delta(t) = "<<setprecision(8)<<time1-time0<<" seconds"<<endl; 
 
}
 
